import fs from 'fs';
import path from 'path';

/**
 * Words example: TS implementation with map/filter/reduce
 */

/**
 * Find all files in the filesystem subtree rooted at folder.
 * @param folder root of subtree, requires folder.isDirectory() == true
 * @return stream of all ordinary files (not folders) that have folder as
 *         their ancestor
 */
function allFilesIn(folder:string):Array<string> {
    let children: Array<string> =
        fs.readdirSync(folder)
        .map(f => path.join(folder, f));
    let descendants: Array<string> = children
                                    .filter(f => fs.lstatSync(f).isDirectory())
                                    .flatMap(allFilesIn);
    return [
        ...descendants,
        ...children.filter(f => fs.lstatSync(f).isFile())
    ];
}

/**
 * Make a filename suffix testing predicate.
 * @param suffix string to test
 * @return a predicate that returns true iff the filename ends with suffix.
 */
function endsWith(suffix:string): (filename:string) => boolean {
    return (filename: string) => filename.endsWith(suffix);
}

let filenames: Array<string> = allFilesIn(".")
                      .filter(s => s.endsWith(".ts"));
let fileContents: Array<Array<string>> = filenames.map(f => { 
    const data = fs.readFileSync(f, { encoding: "utf8", flag: "r" });
    return data.split(/\r?\n/);
});
let lines: Array<string> = fileContents.flatMap(x => x);
let words: Array<string> = lines.flatMap(line => line.split(/\W+/)
                                             .filter(s => s.length > 0));
words.forEach(s => console.log(s));
